from setuptools import setup, find_packages

setup(
    name="df_anonymizer",
    version="0.1.3",
    description="A lightweight library for anonymizing and transforming data in pandas DataFrames, including masking, suppression, perturbation, permutation, generalization, and pseudonymization.",
    autho="Tan Shih Jen",
    packages=find_packages(),
    install_requires=[],
)